/*
*/

#include "filter.h"
#include "capture.h"

#ifdef ALLOC_PRAGMA
#pragma code_seg("PAGE")
#endif // ALLOC_PRAGMA


/* CaptureFilter */

// Public methods

/**
 * The capture filter object constructor.  Since the new operator will
 * have zeroed the memory, do not bother initializing any NULL or 0
 * fields. Only initialize non-NULL, non-0 fields.
 */
CaptureFilter::CaptureFilter(KSFILTER *ksFilter)
{
    PAGED_CODE();

    ASSERT(ksFilter);

    DbgPrint("CaptureFilter::CaptureFilter\n");

    m_ksFilter = ksFilter;
}

CaptureFilter::~CaptureFilter()
{
    PAGED_CODE();
}

// Public static methods

/**
 * This is the filter creation dispatch for the capture filter. It
 * creates the CaptureFilter object, associates it with the AVStream
 * object, and bags it for easy cleanup later.
 * @param ksFilter The AVStream filter being created
 * @param irp The creation Irp
 * @return NT status code.
 */
NTSTATUS
CaptureFilter::DispatchCreate(KSFILTER *ksFilter, IRP *irp)
{
    PAGED_CODE();

    DbgPrint("CaptureFilter::DispatchCreate\n");

    (void)irp;

    CaptureFilter *capFilter = new (NonPagedPool) CaptureFilter(ksFilter);
    if (!capFilter) {
        return STATUS_INSUFFICIENT_RESOURCES;
    }

    NTSTATUS status = ::KsAddItemToObjectBag(ksFilter->Bag,
                                             reinterpret_cast<PVOID>(capFilter),
                                             reinterpret_cast<PFNKSFREE>(CaptureFilter::Cleanup));
    if (NT_SUCCESS(status)) {
        ksFilter->Context = reinterpret_cast<PVOID>(capFilter);
    } else {
        DbgPrint("CaptureFilter::DispatchCreate::KsAddItemToObjectBag - fail\n");
        CaptureFilter::Cleanup(capFilter);
    }

    return status;
}

// Private static methods

/**
 * This is the bag cleanup callback for the CaptureFilter. Not providing
 * one would cause ExFreePool to be used. This is not good for C++
 * constructed objects. We simply delete the object here.
 */
void
CaptureFilter::Cleanup(CaptureFilter *capFilter)
{
    PAGED_CODE();

    DbgPrint("CaptureFilter::Cleanup\n");

    delete capFilter;
}

/* DESCRIPTOR AND DISPATCH LAYOUT */

/**
 */
enum {
    FILTER_CATEGORIES_COUNT = 2,
    CAPTURE_FILTER_PIN_COUNT = 1
};

/**
 *
 */
static
const
GUID
CaptureFilterCategories[FILTER_CATEGORIES_COUNT] = {
    STATICGUIDOF(KSCATEGORY_AUDIO),
    STATICGUIDOF(KSCATEGORY_CAPTURE)
};

/**
 * The list of pin descriptors on the capture filter.
 */
static
const
KSPIN_DESCRIPTOR_EX
CaptureFilterPinDescriptors[CAPTURE_FILTER_PIN_COUNT] = {

    { // Audio Capture Pin
      &CapturePinDispatch,                // Dispatch
      NULL,                               // AutomationTable
      { // PinDescriptor
        0,                              // Interfaces (NULL, 0 == default)
        NULL,
        0,                              // Mediums (NULL, 0 == default)
        NULL,
        0,//SIZEOF_ARRAY(CapturePinDataRanges),// Range Count
        0,//CapturePinDataRanges,           // Ranges
        KSPIN_DATAFLOW_OUT,             // Dataflow
        KSPIN_COMMUNICATION_BOTH,       // Communication
        &KSCATEGORY_AUDIO,              // Category
        NULL,       // Name
        0                               // Reserved
      },
      0,                                // Flags
      1,                                  // Instances Possible
      0,                                  // Instances Necessary
      NULL,                             // Allocator Framing
      reinterpret_cast<PFNKSINTERSECTHANDLEREX>(CapturePin::IntersectHandler)
    }

};

/**
 * This is the dispatch table for the capture filter.  It provides notification
 * of creation, closure, processing (for filter-centrics, not for the capture
 * filter), and resets (for filter-centrics, not for the capture filter).
 */
static
const
KSFILTER_DISPATCH
CaptureFilterDispatch = {
    CaptureFilter::DispatchCreate,    // Filter Create
    NULL,                             // Filter Close
    NULL,                             // Filter Process
    NULL                              // Filter Reset
};

/**
 * The descriptor for the capture filter. We don't specify any topology
 * since there's only one pin on the filter. Realistically, there would
 * be some topological relationships here because there would be input
 * pins from crossbars and the like.
 */
const
KSFILTER_DESCRIPTOR
CaptureFilterDescriptor = {
    &CaptureFilterDispatch,                 // Dispatch Table
    NULL,                                   // Automation Table
    KSFILTER_DESCRIPTOR_VERSION,            // Version
    0,                                      // Flags
    &KSNAME_Filter,                         // Reference GUID
    DEFINE_KSFILTER_PIN_DESCRIPTORS(CaptureFilterPinDescriptors),
    DEFINE_KSFILTER_CATEGORIES(CaptureFilterCategories),
    0,                                      // NodeDescriptorsCount
    sizeof(KSNODE_DESCRIPTOR),              // NodeDescriptorSize
    NULL,                                   // NodeDescriptors
    0,                                      // ConnectionsCount
    NULL,                                   // Connections
    NULL                                    // Component ID
};


